#define WINNT
#define _WIN32_WINNT 0x501

#include "resource.h"
#include <windows.h>
#include <commctrl.h>
#include <psapi.h>
#include <strsafe.h>

#define WM_CHILDEND (WM_APP + 1)
#define WM_UPDATE (WM_APP + 2)

const char mainClass[] = "Main ToolWin";
const int ID_CHECKCAP = 1;
const int ID_CHECKEY = 2;

typedef struct {
	bool valid;
	DWORD style;
} styret;

extern "C"
{
extern const DWORD WinStyleBits[];
extern const DWORD ExWinStyleBits[];

extern const char * ClassNames[];
extern const DWORD * ClassStyles[];
extern const int ClassCount;
}

HWND hwndCap = NULL;
HWND hwndWin = NULL, hwndProc = NULL, hwndGlob = NULL, hwndMisc = NULL;

int ShutdownWindows(UINT type, HWND parentHwnd);
int ReplaceColor(HWND hwnd, HBITMAP hBm, COLORREF oldColor, COLORREF newColor);
int SizeToClient(HWND hwnd, int cx, int cy);

INT_PTR CALLBACK AboutDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
switch(uMsg) {
	case WM_INITDIALOG:
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDOK:
		EndDialog(hwndDlg, IDOK);
		break;
	}
	break;
	case WM_CLOSE:
	EndDialog(hwndDlg, 0);
	break;
	default:
	return false;
}
return true;
}

INT_PTR CALLBACK BitFlagsDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
static DWORD * bits = 0;
switch(uMsg) {
	case WM_INITDIALOG:
	{
		bits = (DWORD *)lParam;
		
		if (bits == WinStyleBits) SetWindowText(hwndDlg, "Window Styles");
		else if (bits == ExWinStyleBits) SetWindowText(hwndDlg, "Extended Window Styles");
		else {
			char className[20];
			GetClassName(hwndCap, className, 20);
			
			char title[35];
			StringCbPrintf(title, 35, "Class Styles (%s)", className);
			
			SetWindowText(hwndDlg, title);
		}
		
		int cols = 3;
		if (bits[0] < 15) cols = 2;
		int i;
		int maxWid = 0;
		
		HDC hdc = GetDC(hwndDlg);
		HGDIOBJ oldFont = SelectObject(hdc, GetStockObject(DEFAULT_GUI_FONT));
		
		for (i=0;i<bits[0];i++) {
			size_t len;
			StringCbLength(((char **)(bits[1]))[i], 30, &len);
			
			SIZE sz;
			GetTextExtentPoint32(hdc, ((char **)(bits[1]))[i], len, &sz);
			
			if (sz.cx > maxWid) maxWid = sz.cx;
		}
		
		for (i=0;i<bits[0];i++) {
			RECT dm;
			
			dm.left = 8 + (i % cols) * 18;
			dm.right = 12;
			dm.top = 6 + (i / cols) * 12;
			dm.bottom = 8;
			
			MapDialogRect(hwndDlg, &dm);
			
			dm.left += (i % cols) * maxWid;
			
			size_t itemLen;
			StringCbLength(((char **)(bits[1]))[i], 30, &itemLen);
			
			SIZE sz;
			GetTextExtentPoint32(hdc, ((char **)(bits[1]))[i], itemLen, &sz);
			
			dm.right += sz.cx;
			
			HWND hwndCtl = CreateWindow("BUTTON", ((char **)(bits[1]))[i], WS_VISIBLE | BS_AUTOCHECKBOX | WS_TABSTOP | WS_CHILD, dm.left, dm.top, dm.right, dm.bottom, hwndDlg, (HMENU)(IDB_BITFLAGBASE + i), GetModuleHandle(NULL), NULL);
			SendMessage(hwndCtl, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), (LPARAM)true);
		}
		
		SelectObject(hdc, oldFont);
		ReleaseDC(hwndDlg, hdc);
		
		RECT dm;
		
		dm.left = 0;
		dm.right = 16 + 6 * (cols - 1) + 12 * cols;
		dm.top = 0;
		dm.bottom = 30 + ((bits[0] - 1) / cols + 1) * 12;
		
		MapDialogRect(hwndDlg, &dm);
		
		dm.right += cols * maxWid;
		
		SizeToClient(hwndDlg, dm.right, dm.bottom);
		
		int winWid = dm.right;
		
		dm.left = 0;
		dm.right = 65;
		dm.top = 10 + ((bits[0] - 1) / cols + 1) * 12;
		dm.bottom = 14;
		
		MapDialogRect(hwndDlg, &dm);
		
		if (dm.right > winWid / 4) dm.right = winWid / 4;
		
		SetWindowPos(GetDlgItem(hwndDlg, IDOK), NULL, (float)(winWid - 2 * dm.right) / 8 * 3, dm.top, dm.right, dm.bottom, SWP_NOZORDER);
		SetWindowPos(GetDlgItem(hwndDlg, IDCANCEL), NULL, (float)(winWid - 2 * dm.right) / 8 * 5 + dm.right, dm.top, dm.right, dm.bottom, SWP_NOZORDER);
		
		unsigned int style;
		if (bits == ExWinStyleBits) style = GetWindowLong(hwndCap, GWL_EXSTYLE);
		else style = GetWindowLong(hwndCap, GWL_STYLE);
		
		for (i=0;i<bits[0];i++) {
			if ((style & bits[i + 2]) == bits[i + 2]) {
				SendDlgItemMessage(hwndDlg, IDB_BITFLAGBASE + i, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
			}
		}
		
		GetWindowRect(hwndDlg, &dm);
		SetWindowPos(hwndDlg, NULL, (GetSystemMetrics(SM_CXSCREEN) - (dm.right - dm.left)) / 2.0, (GetSystemMetrics(SM_CYSCREEN) - (dm.bottom - dm.top)) / 2.0, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	}
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDOK:
		{
			styret * psr = new styret;
			psr->valid = 1;
			psr->style = 0;
			
			int i;
			for (i=0;i<bits[0];i++) {
				if (SendDlgItemMessage(hwndDlg, IDB_BITFLAGBASE + i, BM_GETCHECK, 0, 0) == BST_CHECKED) {
					psr->style |= bits[i + 2];
				}
			}
			
			EndDialog(hwndDlg, (INT_PTR)psr);
		}
		break;
		case IDCANCEL:
		{
			styret * psr = new styret;
			psr->valid = 0;
			
			EndDialog(hwndDlg, (INT_PTR)psr);
		}
		break;
	}
	break;
	case WM_CLOSE:
	{
		styret * psr = new styret;
		psr->valid = 0;
		
		EndDialog(hwndDlg, (INT_PTR)psr);
	}
	break;
	default:
	return false;
}
return true;
}

INT_PTR CALLBACK MiscelDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
switch(uMsg) {
	case WM_INITDIALOG:
	{
		RECT rc;
		GetWindowRect(GetParent(hwndDlg), &rc);
		
		SetWindowPos(hwndDlg, NULL, rc.left, rc.bottom, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	}
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDB_REVPASS:
		SetCapture(hwndDlg);
		break;
	}
	break;
	case WM_LBUTTONDOWN:
	{
		if (GetCapture() != hwndDlg) break;
		
		POINT pt;
		GetCursorPos(&pt);
		HWND wfp = WindowFromPoint(pt);
		
		if (wfp == NULL) break;
		if (!(GetWindowLong(wfp, GWL_STYLE) & ES_PASSWORD)) break;
		
		ReleaseCapture();
		
		PostMessage(wfp, EM_SETPASSWORDCHAR, (WPARAM)0, 0);
		
		SetForegroundWindow(hwndDlg);
	}
	break;
	case WM_CLOSE:
	DestroyWindow(hwndDlg);
	break;
	case WM_DESTROY:
	SendMessage(GetParent(hwndDlg), WM_CHILDEND, (WPARAM)ID_MISC, 0);
	break;
	default:
	return false;
}
return true;
}

INT_PTR CALLBACK GlobalDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
static char keylog[101] = "";
static DWORD thrAttach;
static char prevchar;
static bool enabled = false;
switch(uMsg) {
	case WM_INITDIALOG:
	{
		RECT rc;
		GetWindowRect(GetParent(hwndDlg), &rc);
		
		SetWindowPos(hwndDlg, NULL, rc.left, rc.bottom, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		
		SetDlgItemText(hwndDlg, IDE_KEYLOG, keylog);
		
		if (enabled) {
			SendDlgItemMessage(hwndDlg, IDB_KEYENABLE, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
			EnableWindow(GetDlgItem(hwndDlg, IDE_KEYLOG), true);
			SetTimer(hwndDlg, ID_CHECKEY, 1, NULL);
		}
	}
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDB_KEYENABLE:
		switch(SendDlgItemMessage(hwndDlg, IDB_KEYENABLE, BM_GETCHECK, 0, 0)) {
			case BST_CHECKED:
			EnableWindow(GetDlgItem(hwndDlg, IDE_KEYLOG), true);
			enabled = true;
			
			SetTimer(hwndDlg, ID_CHECKEY, 1, NULL);
			break;
			case BST_UNCHECKED:
			EnableWindow(GetDlgItem(hwndDlg, IDE_KEYLOG), false);
			enabled = false;
			
			KillTimer(hwndDlg, ID_CHECKEY);
			break;
		}
		break;
		case IDB_CLEARKEY:
		SecureZeroMemory(keylog, 101);
		SetDlgItemText(hwndDlg, IDE_KEYLOG, keylog);
		break;
		case IDB_LOGOFF:
		ShutdownWindows(EWX_LOGOFF, hwndDlg);
		break;
		case IDB_RESTART:
		ShutdownWindows(EWX_REBOOT, hwndDlg);
		break;
		case IDB_SHUTDOWN:
		ShutdownWindows(EWX_POWEROFF, hwndDlg);
		break;
	}
	break;
	case WM_TIMER:
	switch (wParam) {
		case ID_CHECKEY:
		{
			AttachThreadInput(GetCurrentThreadId(), thrAttach, false);
			thrAttach = GetWindowThreadProcessId(GetFocus(), NULL);
			AttachThreadInput(GetCurrentThreadId(), thrAttach, true);
			
			BYTE keyState[256] = {0};
			WORD curKey;
			int i;
			int vk = 0;
			int sc;
			
			for (i=0x0D;i<=0xDF;i++) {
				if (i == 0x0E) i = 0x20;
				if (i == 0x21) i = 0x30;
				if (i == 0x3A) i = 0x41;
				if (i == 0x5B) i = 0x60;
				if (i == 0x70) i = 0xBA;
				if (i == 0xC1) i = 0xDB;
				if (GetAsyncKeyState(i) & 0x8000) vk = i;
			}
			
			if (!vk) {
				prevchar = 0;
				break;
			}
			
			if (GetAsyncKeyState(0x10) & 0x8000) keyState[0x10] = 0x80;
			
			sc = MapVirtualKey(vk, 0);
			ToAscii(vk, sc, keyState, &curKey, 0);
			if (prevchar == (char)curKey) break;
			prevchar = (char)curKey;
			
			size_t keyLen;
			StringCbLength(keylog, 101, &keyLen);
			
			if (keyLen == 100) {
				int i;
				for (i=0;i<99;i++) keylog[i] = keylog[i + 1];
				keyLen = 99;
			}
			
			keylog[keyLen] = prevchar;
			keylog[keyLen + 1] = '\0';
			
			SetDlgItemText(hwndDlg, IDE_KEYLOG, keylog);
		}
		break;
	}
	break;
	case WM_CLOSE:
	DestroyWindow(hwndDlg);
	break;
	case WM_DESTROY:
	KillTimer(hwndDlg, ID_CHECKEY);
	SendMessage(GetParent(hwndDlg), WM_CHILDEND, (WPARAM)ID_GLOBTOOLS, 0);
	break;
	default:
	return false;
}
return true;
}

INT_PTR CALLBACK ProcessDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
switch(uMsg) {
	case WM_INITDIALOG:
	{
		RECT rc;
		GetWindowRect(GetParent(hwndDlg), &rc);
		
		SetWindowPos(hwndDlg, NULL, rc.left, rc.bottom, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		
		SendMessage(hwndDlg, WM_UPDATE, 0, 0);
	}
	break;
	case WM_UPDATE:
	{
		DWORD pId;
		GetWindowThreadProcessId(hwndCap, &pId);
		HANDLE hPro = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, false, pId);
		
		DWORD cbNeeded;
		EnumProcessModules(hPro, NULL, 0, &cbNeeded);
		DWORD numMods = cbNeeded / sizeof(HMODULE);
		HMODULE * modules = new HMODULE[numMods];
		EnumProcessModules(hPro, modules, numMods * sizeof(HMODULE), &cbNeeded);
		
		char * procPath = new char[MAX_PATH];
		GetModuleFileNameEx(hPro, modules[0], procPath, MAX_PATH);
		SetDlgItemText(hwndDlg, IDE_PROCPATH, procPath);
		
		char * modNames = new char[50 * numMods];
		ZeroMemory(modNames, 50 * numMods);
		char modName[50];
		
		int i;
		for (i=1;i<numMods;i++) {
			GetModuleBaseName(hPro, modules[i], modName, 50);
			
			if (i < numMods - 1) StringCbCat(modName, 50, ", ");
			StringCbCat(modNames, 50 * numMods, modName);
		}
		
		SetDlgItemText(hwndDlg, IDE_MODNAMES, modNames);
		
		CloseHandle(hPro);
		
		delete [] procPath;
		delete [] modules;
		delete [] modNames;
	}
	break;
	case WM_COMMAND:
	switch (LOWORD(wParam)) {
		case IDB_KILLPRO:
		{
			if (MessageBox(hwndDlg, "Forcefully terminating a process could cause system instability.\r\nAre you sure you want to continue?", "Warning", MB_ICONWARNING | MB_YESNO) != IDYES) break;
			
			DWORD pId;
			GetWindowThreadProcessId(hwndCap, &pId);
			
			HANDLE hPro = OpenProcess(PROCESS_TERMINATE, false, pId);
			if (hPro == NULL) break;
			
			TerminateProcess(hPro, 1);
			
			CloseHandle(hPro);
		}
		break;
		case IDB_KILLTHR:
		{
			if (MessageBox(hwndDlg, "Forcefully terminating a thread could cause system instability.\r\nAre you sure you want to continue?", "Warning", MB_ICONWARNING | MB_YESNO) != IDYES) break;
			
			DWORD tId = GetWindowThreadProcessId(hwndCap, NULL);
			
			HANDLE hThr = OpenThread(THREAD_TERMINATE, false, tId);
			if (hThr == NULL) break;
			
			TerminateThread(hThr, 1);
			
			CloseHandle(hThr);
		}
		break;
	}
	break;
	case WM_CLOSE:
	DestroyWindow(hwndDlg);
	break;
	case WM_DESTROY:
	SendMessage(GetParent(hwndDlg), WM_CHILDEND, (WPARAM)ID_PROCTOOLS, 0);
	break;
	default:
	return false;
}
return true;
}

INT_PTR CALLBACK WindowDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
static POINT pt1;
static bool phaseTwo;
switch(uMsg) {
	case WM_INITDIALOG:
	{
		RECT rc;
		GetWindowRect(GetParent(hwndDlg), &rc);
		
		SetWindowPos(hwndDlg, NULL, rc.left, rc.bottom, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		
		SendMessage(hwndDlg, WM_UPDATE, 0, 0);
		SendDlgItemMessage(hwndDlg, IDU_ALPHA, UDM_SETRANGE, 0, (LPARAM)MAKELONG(255, 0));
	}
	break;
	case WM_UPDATE:
	{
		char className[20];
		GetClassName(hwndCap, className, 20);
		SetDlgItemText(hwndDlg, IDE_CLASS, className);
		
		bool control = false;
		int i;
		
		for (i=0;i<ClassCount;i++) {
			if (lstrcmpi(className, ClassNames[i]) == 0) control = true;
		}
		
		if (control) EnableWindow(GetDlgItem(hwndDlg, IDB_CLASSTY), true);
		else EnableWindow(GetDlgItem(hwndDlg, IDB_CLASSTY), false);
		
		if (!((GetWindowLong(hwndCap, GWL_STYLE) & WS_CHILD) == WS_CHILD)) {
			EnableWindow(GetDlgItem(hwndDlg, IDB_CHOP), true);
			EnableWindow(GetDlgItem(hwndDlg, IDB_UNCHOP), true);
		} else {
			EnableWindow(GetDlgItem(hwndDlg, IDB_CHOP), false);
			EnableWindow(GetDlgItem(hwndDlg, IDB_UNCHOP), false);
		}
		
		OSVERSIONINFO ovi;
		ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		
		GetVersionEx(&ovi);
		
		if (ovi.dwPlatformId == VER_PLATFORM_WIN32_NT && ovi.dwMajorVersion >= 5 && !((GetWindowLong(hwndCap, GWL_STYLE) & WS_CHILD) == WS_CHILD)) {
			EnableWindow(GetDlgItem(hwndDlg, IDE_ALPHA), true);
			EnableWindow(GetDlgItem(hwndDlg, IDU_ALPHA), true);
			
			BYTE alpha = 255;
			if (!GetLayeredWindowAttributes(hwndCap, NULL, &alpha, NULL)) {
				if (GetLastError() != ERROR_INVALID_PARAMETER) MessageBox(hwndDlg, "Unknown error finding current alpha value.\r\nDefaulting to 255.", "Error", MB_ICONERROR | MB_OK);
			}
			
			SetDlgItemInt(hwndDlg, IDE_ALPHA, alpha, false);
		} else {
			EnableWindow(GetDlgItem(hwndDlg, IDE_ALPHA), false);
			EnableWindow(GetDlgItem(hwndDlg, IDU_ALPHA), false);
			SetDlgItemInt(hwndDlg, IDE_ALPHA, 255, false);
		}
	}
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDB_SHOW:
		ShowWindowAsync(hwndCap, SW_SHOW);
		break;
		case IDB_HIDE:
		ShowWindowAsync(hwndCap, SW_HIDE);
		break;
		case IDB_MINIMIZE:
		ShowWindowAsync(hwndCap, SW_MINIMIZE);
		break;
		case IDB_MAXIMIZE:
		ShowWindowAsync(hwndCap, SW_MAXIMIZE);
		break;
		case IDB_RESTORE:
		ShowWindowAsync(hwndCap, SW_RESTORE);
		break;
		case IDB_CLOSE:
		PostMessage(hwndCap, WM_CLOSE, 0, 0);
		break;
		case IDB_MINIMIZEDI:
		ShowWindowAsync(hwndCap, SW_FORCEMINIMIZE);
		break;
		case IDB_DESTROY:
		EndTask(hwndCap, 0, true);
		break;
		case IDB_WINSTY:
		{
			styret * psr = (styret*)DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_BITFLAGS), hwndDlg, BitFlagsDlgProc, (LPARAM)WinStyleBits);
			
			if (psr->valid) {
				if ((psr->style & WS_VISIBLE) == WS_VISIBLE) ShowWindow(hwndCap, SW_SHOW);
				else ShowWindow(hwndCap, SW_HIDE);
				if ((psr->style & WS_DISABLED) == WS_DISABLED) EnableWindow(hwndCap, false);
				else EnableWindow(hwndCap, true);
				
				SetWindowLongPtr(hwndCap, GWL_STYLE, psr->style);
				SetWindowPos(hwndCap, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED | SWP_ASYNCWINDOWPOS);
			}
			
			delete psr;
		}
		break;
		case IDB_EXWINSTY:
		{
			styret * psr = (styret*)DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_BITFLAGS), hwndDlg, BitFlagsDlgProc, (LPARAM)ExWinStyleBits);
			
			if (psr->valid) {
				if ((psr->style & WS_EX_TOPMOST) == WS_EX_TOPMOST) SetWindowPos(hwndCap, HWND_TOPMOST, 0, 0, 0, 0, SWP_ASYNCWINDOWPOS | SWP_NOSIZE | SWP_NOMOVE);
				else SetWindowPos(hwndCap, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_ASYNCWINDOWPOS | SWP_NOSIZE | SWP_NOMOVE);
				
				SetWindowLongPtr(hwndCap, GWL_EXSTYLE, psr->style);
				SetWindowPos(hwndCap, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED | SWP_ASYNCWINDOWPOS);
			}
			
			delete psr;
		}
		break;
		case IDB_CLASSTY:
		{
			const DWORD * classBits;
			
			char className[20];
			GetClassName(hwndCap, className, 20);
			
			int i;
			for (i=0;i<ClassCount;i++) {
				if (lstrcmpi(className, ClassNames[i]) == 0) {
					classBits = ClassStyles[i];
					break;
				}
			}
			
			styret * psr = (styret*)DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_BITFLAGS), hwndDlg, BitFlagsDlgProc, (LPARAM)classBits);
			
			if (psr->valid) {
				SetWindowLongPtr(hwndCap, GWL_STYLE, psr->style);
				SetWindowPos(hwndCap, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED | SWP_ASYNCWINDOWPOS);
			}
			
			delete psr;
		}
		break;
		case IDB_CHOP:
		SetCapture(hwndDlg);
		break;
		case IDB_UNCHOP:
		SetWindowRgn(hwndCap, NULL, true);
		break;
		case IDE_ALPHA:
		switch (HIWORD(wParam)) {
			case EN_CHANGE:
			if (IsWindowVisible(hwndDlg) && IsWindowEnabled(GetDlgItem(hwndDlg, IDE_ALPHA))) {
				if (GetDlgItemInt(hwndDlg, IDE_ALPHA, NULL, false) == 255) {
					SetWindowLong(hwndCap, GWL_EXSTYLE, GetWindowLong(hwndCap, GWL_EXSTYLE) & ~WS_EX_LAYERED);
				} else {
					SetWindowLong(hwndCap, GWL_EXSTYLE, GetWindowLong(hwndCap, GWL_EXSTYLE) | WS_EX_LAYERED);
					
					SetLayeredWindowAttributes(hwndCap, 0, GetDlgItemInt(hwndDlg, IDE_ALPHA, NULL, false), LWA_ALPHA);
				}
			}
			break;
		}
		break;
	}
	break;
	case WM_LBUTTONDOWN:
	if (GetCapture() != hwndDlg) break;
	
	if (!phaseTwo) {
		GetCursorPos(&pt1);
		
		if (GetAncestor(WindowFromPoint(pt1), GA_ROOTOWNER) != hwndCap) {
			ReleaseCapture();
			break;
		}
		
		RECT rc;
		GetWindowRect(hwndCap, &rc);
		
		pt1.x -= rc.left;
		pt1.y -= rc.top;
		
		phaseTwo = true;
		SetDlgItemText(hwndDlg, IDB_CHOP, "Chop: P2");
	} else {
		POINT pt2;
		GetCursorPos(&pt2);
		
		if (GetAncestor(WindowFromPoint(pt2), GA_ROOTOWNER) != hwndCap) {
			ReleaseCapture();
			phaseTwo = false;
			SetDlgItemText(hwndDlg, IDB_CHOP, "Chop");
			break;
		}
		
		RECT rc;
		GetWindowRect(hwndCap, &rc);
		
		pt2.x -= rc.left;
		pt2.y -= rc.top;
		
		HRGN hrgnWin = CreateRectRgn(0, 0, 0, 0);
		GetWindowRgn(hwndCap, hrgnWin);
		
		HRGN hrgnClip = CreateRectRgn(pt1.x, pt1.y, pt2.x, pt2.y);
		
		HRGN hrgnWinNew = CreateRectRgn(0, 0, 0, 0);
		CombineRgn(hrgnWinNew, hrgnWin, hrgnClip, RGN_XOR);
		
		SetWindowRgn(hwndCap, hrgnWinNew, true);
		
		ReleaseCapture();
		
		DeleteObject(hrgnWin);
		DeleteObject(hrgnClip);
		DeleteObject(hrgnWinNew);
		
		phaseTwo = false;
		SetDlgItemText(hwndDlg, IDB_CHOP, "Chop");
	}
	break;
	case WM_CLOSE:
	DestroyWindow(hwndDlg);
	break;
	case WM_DESTROY:
	SendMessage(GetParent(hwndDlg), WM_CHILDEND, (WPARAM)ID_WINTOOLS, 0);
	break;
	default:
	return false;
}
return true;
}

LRESULT CALLBACK MainWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
switch(uMsg) {
	case WM_CREATE:
	{
		HWND rBar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | CCS_NODIVIDER, 0, 0, 0, 0, hwnd, (HMENU)IDR_TOOLS, GetModuleHandle(NULL), NULL);
		
		REBARBANDINFO rbbi;
		rbbi.cbSize = sizeof(REBARBANDINFO);
		rbbi.fMask  = RBBIM_STYLE | RBBIM_CHILD | RBBIM_CHILDSIZE;
		rbbi.fStyle = RBBS_NOGRIPPER;
		
		HWND hTool = CreateWindow(TOOLBARCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | TBSTYLE_LIST | TBSTYLE_FLAT | CCS_NODIVIDER | CCS_NORESIZE, 0, 0, 0, 0, hwnd, (HMENU)IDT_WINCTL, GetModuleHandle(NULL), NULL);
		HWND hTool2 = CreateWindow(TOOLBARCLASSNAME, NULL, WS_CHILD | WS_VISIBLE | TBSTYLE_LIST | TBSTYLE_FLAT | CCS_NODIVIDER | CCS_NORESIZE, 0, 0, 0, 0, hwnd, (HMENU)IDT_APPCTL, GetModuleHandle(NULL), NULL);
		
		SendMessage(hTool, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);
		SendMessage(hTool2, TB_BUTTONSTRUCTSIZE, (WPARAM)sizeof(TBBUTTON), 0);
		
		HBITMAP toolImg = (HBITMAP)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IRB_TOOLIMG), IMAGE_BITMAP, 0, 0, LR_SHARED);
		
		ReplaceColor(hwnd, toolImg, RGB(192, 192, 192), GetSysColor(COLOR_BTNFACE));
		
		TBADDBITMAP tbab;
		
		tbab.hInst = NULL;
		tbab.nID = (UINT_PTR)toolImg;
		
		SendMessage(hTool, TB_SETBITMAPSIZE, 0, (LPARAM)MAKELONG(24, 24));
		SendMessage(hTool2, TB_SETBITMAPSIZE, 0, (LPARAM)MAKELONG(24, 24));
		
		SendMessage(hTool, TB_ADDBITMAP, 9, (LPARAM)&tbab);
		SendMessage(hTool2, TB_ADDBITMAP, 9, (LPARAM)&tbab);
		
		TBBUTTON tbb[5];
		
		ZeroMemory(tbb, sizeof(tbb));
		
		tbb[0].iBitmap = 0;
		tbb[0].fsState = 0;
		tbb[0].fsStyle = BTNS_AUTOSIZE | BTNS_CHECK;
		tbb[0].idCommand = ID_WINTOOLS;
		tbb[0].iString = (INT_PTR)"Window Tools";
		
		tbb[1].iBitmap = 1;
		tbb[1].fsState = 0;
		tbb[1].fsStyle = BTNS_AUTOSIZE | BTNS_CHECK;
		tbb[1].idCommand = ID_PROCTOOLS;
		tbb[1].iString = (INT_PTR)"Process/Thread Tools";
		
		tbb[2].iBitmap = 2;
		tbb[2].fsState = TBSTATE_ENABLED;
		tbb[2].fsStyle = BTNS_AUTOSIZE | BTNS_CHECK;
		tbb[2].idCommand = ID_GLOBTOOLS;
		tbb[2].iString = (INT_PTR)"Global Tools";
		
		tbb[3].iBitmap = 3;
		tbb[3].fsState = TBSTATE_ENABLED;
		tbb[3].fsStyle = BTNS_AUTOSIZE | BTNS_CHECK;
		tbb[3].idCommand = ID_MISC;
		tbb[3].iString = (INT_PTR)"Miscellaneous";
		
		SendMessage(hTool, TB_ADDBUTTONS, (WPARAM)4, (LPARAM)&tbb);
		
		rbbi.hwndChild = hTool;
		rbbi.cyMinChild = HIWORD(SendMessage(hTool, TB_GETBUTTONSIZE, 0, 0)) + 1;
		
		SendMessage(rBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbbi);
		
		ZeroMemory(tbb, sizeof(tbb));
		
		tbb[0].iBitmap = 4;
		tbb[0].fsState = TBSTATE_ENABLED;
		tbb[0].fsStyle = BTNS_AUTOSIZE;
		tbb[0].idCommand = ID_ACQUIRE;
		tbb[0].iString = (INT_PTR)"Acquire";
		
		tbb[1].iBitmap = 5;
		tbb[1].fsState = TBSTATE_ENABLED;
		tbb[1].fsStyle = BTNS_AUTOSIZE | BTNS_CHECK;
		tbb[1].idCommand = ID_ROOT;
		tbb[1].iString = (INT_PTR)"Root Only";
		
		tbb[2].iBitmap = 6;
		tbb[2].fsState = TBSTATE_ENABLED;
		tbb[2].fsStyle = BTNS_AUTOSIZE;
		tbb[2].idCommand = ID_HELP;
		tbb[2].iString = (INT_PTR)"Help";
		
		tbb[3].iBitmap = 7;
		tbb[3].fsState = TBSTATE_ENABLED;
		tbb[3].fsStyle = BTNS_AUTOSIZE;
		tbb[3].idCommand = ID_ABOUT;
		tbb[3].iString = (INT_PTR)"About";
		
		tbb[4].iBitmap = 8;
		tbb[4].fsState = TBSTATE_ENABLED;
		tbb[4].fsStyle = BTNS_AUTOSIZE;
		tbb[4].idCommand = ID_EXIT;
		tbb[4].iString = (INT_PTR)"Exit";
		
		SendMessage(hTool2, TB_ADDBUTTONS, (WPARAM)5, (LPARAM)&tbb);
		
		rbbi.fStyle = RBBS_NOGRIPPER | RBBS_BREAK;
		rbbi.hwndChild = hTool2;
		
		SendMessage(rBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbbi);
		
		TBMETRICS tbm;
		tbm.cbSize = sizeof(TBMETRICS);
		
		tbm.dwMask = TBMF_BUTTONSPACING;
		tbm.cxButtonSpacing = 1;
		tbm.cyButtonSpacing = 0;
		
		SendMessage(hTool, TB_SETMETRICS, 0, (LPARAM)&tbm);
		SendMessage(hTool2, TB_SETMETRICS, 0, (LPARAM)&tbm);
		
		SIZE sz;
		SendMessage(hTool, TB_GETMAXSIZE, 0, (LPARAM)&sz);
		
		SizeToClient(hwnd, sz.cx, SendMessage(rBar, RB_GETBARHEIGHT, 0, 0));
	}
	break;
	case WM_CHILDEND:
	SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_SETSTATE, wParam, (LPARAM)TBSTATE_ENABLED);
	break;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case ID_WINTOOLS:
		DestroyWindow(hwndWin);
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_GETSTATE, (WPARAM)ID_WINTOOLS, 0) & TBSTATE_CHECKED) {
			hwndWin = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_WINDOW), hwnd, WindowDlgProc);
			ShowWindow(hwndWin, SW_SHOW);
		}
		break;
		case ID_PROCTOOLS:
		DestroyWindow(hwndProc);
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_GETSTATE, (WPARAM)ID_PROCTOOLS, 0) & TBSTATE_CHECKED) {
			hwndProc = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_PROCESS), hwnd, ProcessDlgProc);
			ShowWindow(hwndProc, SW_SHOW);
		}
		break;
		case ID_GLOBTOOLS:
		DestroyWindow(hwndGlob);
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_GETSTATE, (WPARAM)ID_GLOBTOOLS, 0) & TBSTATE_CHECKED) {
			hwndGlob = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_GLOBAL), hwnd, GlobalDlgProc);
			ShowWindow(hwndGlob, SW_SHOW);
		}
		break;
		case ID_MISC:
		DestroyWindow(hwndMisc);
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_GETSTATE, (WPARAM)ID_MISC, 0) & TBSTATE_CHECKED) {
			hwndMisc = CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_MISCEL), hwnd, MiscelDlgProc);
			ShowWindow(hwndMisc, SW_SHOW);
		}
		break;
		case ID_ACQUIRE:
		{
			TBBUTTONINFO tbi;
			
			tbi.cbSize = sizeof(TBBUTTONINFO);
			tbi.dwMask = TBIF_TEXT;
			
			tbi.pszText = "Acquiring...";
			
			SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_APPCTL, TB_SETBUTTONINFO, (WPARAM)ID_ACQUIRE, (LPARAM)&tbi);
			
			SetCapture(hwnd);
		}
		break;
		case ID_ROOT:
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_APPCTL, TB_GETSTATE, (WPARAM)ID_ROOT, 0) & TBSTATE_CHECKED) {
			hwndCap = GetAncestor(hwndCap, GA_ROOTOWNER);
			
			PostMessage(hwndWin, WM_UPDATE, 0, 0);
			PostMessage(hwndProc, WM_UPDATE, 0, 0);
		}
		break;
		case ID_HELP:
		ShellExecute(hwnd, "open", "help.chm", NULL, NULL, SW_SHOWNORMAL);
		break;
		case ID_ABOUT:
		DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IRD_ABOUT), hwnd, AboutDlgProc);
		break;
		case ID_EXIT:
		DestroyWindow(hwnd);
		break;
	}
	break;
	case WM_TIMER:
	switch(wParam) {
		case ID_CHECKCAP:
		if (!IsWindow(hwndCap)) {
			hwndCap = NULL;
			
			DestroyWindow(hwndWin);
			DestroyWindow(hwndProc);
			
			SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_ENABLEBUTTON, (WPARAM)ID_WINTOOLS, (LPARAM)0);
			SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_ENABLEBUTTON, (WPARAM)ID_PROCTOOLS, (LPARAM)0);
			
			KillTimer(hwnd, ID_CHECKCAP);
		}
		break;
	}
	break;
	case WM_LBUTTONUP:
	{
		if (GetCapture() != hwnd) break;
		
		POINT pt;
		GetCursorPos(&pt);
		
		HWND wfp = WindowFromPoint(pt);
		
		if (SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_APPCTL, TB_GETSTATE, (WPARAM)ID_ROOT, 0) & TBSTATE_CHECKED)
			wfp = GetAncestor(wfp, GA_ROOTOWNER);
		
		if (GetAncestor(wfp, GA_ROOTOWNER) == hwnd || wfp == NULL) break;
		
		ShowWindowAsync(hwndCap, SW_SHOW);
		
		hwndCap = wfp;
		
		ReleaseCapture();
		
		SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_ENABLEBUTTON, (WPARAM)ID_WINTOOLS, (LPARAM)1);
		SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_WINCTL, TB_ENABLEBUTTON, (WPARAM)ID_PROCTOOLS, (LPARAM)1);
		
		PostMessage(hwndWin, WM_UPDATE, 0, 0);
		PostMessage(hwndProc, WM_UPDATE, 0, 0);
		
		SetTimer(hwnd, ID_CHECKCAP, 50, NULL);
		
		TBBUTTONINFO tbi;
		
		tbi.cbSize = sizeof(TBBUTTONINFO);
		tbi.dwMask = TBIF_TEXT;
		
		tbi.pszText = "Acquire";
		
		SendDlgItemMessage(GetDlgItem(hwnd, IDR_TOOLS), IDT_APPCTL, TB_SETBUTTONINFO, (WPARAM)ID_ACQUIRE, (LPARAM)&tbi);
	}
	break;
	case WM_CLOSE:
	DestroyWindow(hwnd);
	break;
	case WM_DESTROY:
	ShowWindowAsync(hwndCap, SW_SHOW);
	
	PostQuitMessage(0);
	break;
	default:
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}
return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
	INITCOMMONCONTROLSEX icc;
	
	icc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icc.dwICC = ICC_STANDARD_CLASSES | ICC_BAR_CLASSES | ICC_COOL_CLASSES | ICC_UPDOWN_CLASS;
	
	InitCommonControlsEx(&icc);
	
	CreateMutex(NULL, true, "sherlockuniquemutex");
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		SetForegroundWindow(FindWindow(mainClass, NULL));
		
		return 0;
	}
	
	WNDCLASSEX wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.lpfnWndProc = MainWindowProc;
	wc.hInstance = hInstance;
	wc.hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IRI_MAIN), IMAGE_ICON, 32, 32, 0);
	wc.hCursor = (HCURSOR)LoadImage(NULL, IDC_ARROW, IMAGE_CURSOR, 0, 0, 0);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
	wc.lpszClassName = mainClass;
	wc.hIconSm = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IRI_MAIN), IMAGE_ICON, 16, 16, 0);
	
	if(!RegisterClassEx(&wc)) {
		MessageBox(NULL, "Window Registration Failed! Exiting.", "Error", MB_ICONERROR | MB_OK);
		
		return 0;
	}
	
	HWND hwnd;
	hwnd = CreateWindowEx(WS_EX_WINDOWEDGE | WS_EX_TOPMOST, mainClass, "Sherlock", WS_OVERLAPPED | WS_SYSMENU | WS_CAPTION | WS_MINIMIZEBOX, GetSystemMetrics(SM_CXSCREEN) / 10.0, GetSystemMetrics(SM_CYSCREEN) / 10.0, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);
	
	if(hwnd == NULL) {
		MessageBox(NULL, "Window Creation Failed! Exiting.", "Error", MB_ICONERROR | MB_OK);
		
		return 0;
	}
	
	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);
	
	MSG Msg;
	
	while(GetMessage(&Msg, NULL, 0, 0) > 0) {
		if(!(IsDialogMessage(hwndWin, &Msg) || IsDialogMessage(hwndProc, &Msg) || IsDialogMessage(hwndGlob, &Msg) || IsDialogMessage(hwndMisc, &Msg))) {
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}
	}
	
	return Msg.wParam;
}

int ShutdownWindows(UINT type, HWND parentHwnd) {
char buffer[11];

if (type == EWX_LOGOFF) StringCbCopy(buffer, 11, "log out of");
else if (type == EWX_REBOOT) StringCbCopy(buffer, 11, "restart");
else if (type == EWX_POWEROFF) StringCbCopy(buffer, 11, "shutdown");
else return 0;

char buffer2[45];
StringCbPrintf(buffer2, 45, "Are you sure you want to %s Windows?", buffer);

if (MessageBox(parentHwnd, buffer2, "Alert", MB_ICONQUESTION | MB_YESNO) != IDYES) return 0;

OSVERSIONINFO ovi;
ovi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
GetVersionEx(&ovi);

if (ovi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
HANDLE hToken;
TOKEN_PRIVILEGES tkp;

if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) {
MessageBox(NULL, "Couldn't open process token!", "Error", MB_ICONEXCLAMATION | MB_OK);
return 0;
}

LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);

tkp.PrivilegeCount = 1;
tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

AdjustTokenPrivileges(hToken, false, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);
if (GetLastError() != ERROR_SUCCESS) {
MessageBox(NULL, "Couldn't adjust token privileges!", "Error", MB_ICONEXCLAMATION | MB_OK);
return 0;
}}
ExitWindowsEx(type, 0);
return 0;
}

int ReplaceColor(HWND hwnd, HBITMAP hBm, COLORREF oldColor, COLORREF newColor) {
	HDC hdc = GetDC(hwnd);
	HDC hdcMem = CreateCompatibleDC(hdc);
	
	HGDIOBJ hgoBi = SelectObject(hdcMem, hBm);
	
	BITMAP bm;
	GetObject(hBm, sizeof(BITMAP), &bm);
	
	int i;
	for (i=0;i<bm.bmWidth*bm.bmHeight;i++) {
		if (GetPixel(hdcMem, i % bm.bmWidth, i / bm.bmWidth) == oldColor) {
			SetPixel(hdcMem, i % bm.bmWidth, i / bm.bmWidth, newColor);
		}
	}
	
	SelectObject(hdcMem, hgoBi);
	
	DeleteDC(hdcMem);
	ReleaseDC(hwnd, hdc);
	
	return 0;
}

int SizeToClient(HWND hwnd, int cx, int cy) {
	RECT rcW, rcCl;
	
	GetWindowRect(hwnd, &rcW);
	GetClientRect(hwnd, &rcCl);
	
	int wCx = ((rcW.right - rcW.left) - rcCl.right) + cx;
	int wCy = ((rcW.bottom - rcW.top) - rcCl.bottom) + cy;
	
	SetWindowPos(hwnd, NULL, 0, 0, wCx, wCy, SWP_NOZORDER | SWP_NOMOVE);
	
	return 0;
}
